from .imports import *
def eatInner(string: str, list_objects:(str or list)) -> any:
    """
    Removes characters from the inner part of a string or list.

    Args:
        x (str or list): The input string or list.
        ls (list): The list of characters to remove.

    Returns:
        any: The modified string or list.
    """
    if not isinstance(list_objects,list):
        list_objects = [list_objects]
    if not isinstance(string,str):
        string = str(string)
    if string and list_objects:
        for char in string:
            if string:
                if char not in list_objects:
                    return string
                string = string[1:]
    return string


def eatOuter(string: str, list_objects:(str or list)) -> any:
    """
    Removes characters from the outer part of a string or list.

    Args:
        x (str or list): The input string or list.
        ls (list): The list of characters to remove.

    Returns:
        any: The modified string or list.
    """
    if not isinstance(list_objects,list):
        list_objects = [list_objects]
    if not isinstance(string,str):
        string = str(string)
    if string and list_objects:
        for i in range(len(string)):
            if string:
                if string[-1] not in list_objects:
                    return string
                string = string[:-1]
    return string
def eatAll(string: str, list_objects:(str or list)) -> any:
    """
    Removes characters from both the inner and outer parts of a string or list.

    Args:
        x (str or list): The input string or list.
        ls (list): The list of characters to remove.

    Returns:
        any: The modified string or list.
    """
    if not isinstance(list_objects,list):
        list_objects = [list_objects]
    if not isinstance(string,str):
        string = str(string)
    if string and list_objects:
        string = eatInner(string, list_objects)
    if string and list_objects:
        string = eatOuter(string, list_objects)
    return string



def eatElse(
    stringObj,
    chars=None,
    ints=True,
    alpha=True,
    lower=True,
    capitalize=True,
    string=True,
    listObj=True
):
    alpha_ints = get_alpha_ints(
        ints=True,
        alpha=True,
        lower=True,
        capitalize=True,
        string=True,
        listObj=True
        )
    chars = make_list(chars or [])+alpha_ints

    while True:
        if stringObj:
            str_0 = stringObj[0] not in chars
            str_1 = stringObj[-1] not in chars
            str_eat = str_0 or str_1
            if not str_eat:
                return stringObj
            if stringObj and str_0:
                stringObj = stringObj[1:] if len(stringObj) !=1 else ""
            if stringObj and str_1:
                stringObj = stringObj[:-1] if len(stringObj) !=1 else ""   
        else:
            return stringObj



from .imports import *
def get_alpha():
    return 'abcdefghijklmnopqrstuvwxyz'
def get_alpha_list() -> list:
    """
    Generates a list of all lowercase alphabets.
    
    Returns:
        list: A list of all lowercase alphabets.
    """
    return list(get_alpha())
def is_alpha(char,case_sensative=False):
    alphas = get_alpha()
    if not case_sensative:
        alphas+=alphas.upper()
    return char in alphas
def find_it_alph(ls: list, y: any) -> int:
    """
    Finds the index of an element in a list.

    Args:
        ls (list): The list to search.
        y (any): The element to find.

    Returns:
        int: The index of the element in the list. If not found, returns -1.
    """
    i = 0
    while str(ls[i]) != str(y):
        i += 1
    return i
##def get_alpha(k: Union[int,float]) -> str:
##    """
##    Retrieves the alphabetic character corresponding to the given index.
##
##    Args:
##        k (int|float): The index of the character.
##
##    Returns:
##        str: The alphabetic character.
##    """
##    k, 
##    if k <= len(get_alpha_list()):
##        return 0, k
##    mul = int(float(k) / float(len(get_alpha_list())))
##    rem = int(k) - int(mul * len(get_alpha_list()))
##    if mul - 1 > -1:
##        return str(alph[mul]) + str(alph[rem])
##    return str(alph[rem])
def getAlphas(lower=True,capitalize=False,listObj=False):
    obj = ''
    alphas = get_alpha()
    if lower:
        obj+=alphas
    if capitalize:
        obj+=alphas.upper()
    if listObj:
        obj = list(obj)
    return obj



from .imports import *
class abstractEnv:
    def __init__(self,key='MY_PASSWORD',file_name=None,path=os.getcwd(),deep_scan=False):
        file_name = file_name or '.env'
        self.re_initialize(file_name=file_name,key=key,path=path,deep_scan=deep_scan)
    def re_initialize(self,key='MY_PASSWORD',file_name='.env',path=os.getcwd(),deep_scan=False):
        """
        Initializes an AbstractEnv object to manage environment variables.

        Args:
            key (str, optional): The key to search for in the .env file. Defaults to 'MY_PASSWORD'.
            file_name (str, optional): The name of the .env file. Defaults to '.env'.
            path (str, optional): The path where the .env file is located. Defaults to the current working directory.
        """

        self.key = key or 'MY_PASSWORD'

        file_name = file_name or '.env'
        self.key = key or 'MY_PASSWORD'
        self.deep_scan=deep_scan
        self.current_folder = os.getcwd()
        if path and os.path.isfile(path):
            self.file_name = os.path.basename(path)
            self.path = os.path.dirname(path)
        else:
            self.path = path or self.current_folder
            self.file_name = file_name or '.env'
        self.start_path_env = os.path.join(self.path,self.file_name)
        self.home_folder = os.path.expanduser("~")
        self.envy_all = os.path.join(self.home_folder,'.envy_all')
        self.directories = self.get_directories()
        self.env_value = self.find_and_read_env_file(key=self.key,file_name=self.file_name, path=self.path,initialize=False)

    def find_and_read_env_file(self,key:str=None, file_name:str=None, path=None,initialize=True,deep_scan=False):
        """
        Search for an environment file and read a specific key from it.

        Args:
            file_name (str): Name of the .env file to be searched. Defaults to '.env'.
            key (str): Key to be retrieved from the .env file. Defaults to 'MY_PASSWORD'.
            start_path (str): Directory path to start the search from. If None, search starts from current directory. 

        Returns:
            str: The value corresponding to the key if found, otherwise None.
        """
        # Set the default start_path to the current directory if it's None
        # Try to find the file in the start_path
        key = key or self.key
        path = path or self.start_path_env
        file_name = file_name or self.file_name
        if initialize:
            self.re_initialize(key=key,file_name=file_name,path=path)
        for directory in self.directories:
            if directory and os.path.isdir(directory) and self.file_name:
                env_path = os.path.join(directory,self.file_name)
                if os.path.isfile(env_path):
                    value = self.search_for_env_key(key=key,path=env_path)
                self.env_path = os.path.join(directory,self.file_name)
                if os.path.isfile(self.env_path):
                    value = self.search_for_env_key(key=key,path=self.env_path,deep_scan=deep_scan)
                    if value:
                        return value
    def get_directories(self):
        """
        Retrieves a list of directories to search for the .env file.

        Returns:
            list: A list of directories including the specified path, current folder, home folder, and '.envy_all' directory.
        """
        directories=[]
        for directory in [self.path,self.current_folder,self.home_folder,self.envy_all]:
            if os.path.isdir(directory) and directory not in directories:
                directories.append(directory)
        return directories
    

    def search_for_env_key(self,key:str=None,path:str=None,deep_scan=False):

        """
        Retrieves the value of a specified environment variable from a .env file.

        Args:
            key (str, optional): The key to search for in the .env file. Defaults to None.
            path (str, optional): The path to the .env file. Defaults to None.
            file_name (str, optional): The name of the .env file. Defaults to None.

        Returns:
            str: The value of the environment variable if found, otherwise None.
        """

        highest = [None,0,0.0]
        key = key or self.default_env_key
        path = path or self.start_path_env
        if path and os.path.isfile(path):
            with open(path, "r") as f:
                for line in f:
                    line_key,line_value = split_eq(line)
                    # If the line contains the key, return the value after stripping extra characters
                    if line_key == key:
                        return line_value
                    if deep_scan:
                        line_keys = []
                        key_parts=0
                        for key_part in key.split('_'):
                            line_keys.append([])
                            if key_part in line_key:
                                line_keys[-1].append(line_key)
                                key_parts+=len(key_part)
                        if float(key_parts/len(key))>=0.5 and key_parts >highest[1]:
                            highest = [line_value,key_parts]
                    if deep_scan and highest[0] != None:
                        return line_value
    def get_env_value(key:str=None,path:str=os.getcwd(),file_name:str=None):
        """
        Retrieves the value of the specified environment variable.

        Args:
            path (str): The path to the environment file. Defaults to None.
            file_name (str): The name of the environment file. Defaults to '.env'.
            key (str): The key to search for in the .env file. Defaults to 'MY_PASSWORD'.

        Returns:
            str: The value of the environment variable if found, otherwise None.
        """
        if safe_env_load(path):
            return os.getenv(key)
        return find_and_read_env_file(file_name=file_name, key=key, path=path_ls)

AbstractEnv = abstractEnv



from .imports import *
def get_logFile(bpName: str = None, maxBytes: int = 100_000, backupCount: int = 3) -> logging.Logger:
    """
    If bpName is None, use the “caller module’s basename” as the logger name.
    Otherwise, use the explicitly provided bpName.
    """
    if bpName is None:
        # Find the first frame outside logging_utils.py
        frame_idx = _find_caller_frame_index()
        frame_info = inspect.stack()[frame_idx]
        caller_path = frame_info.filename  # e.g. "/home/joe/project/app/routes.py"
        bpName = os.path.splitext(os.path.basename(caller_path))[0]
        del frame_info



    logger = logging.getLogger(bpName)
    logger.setLevel(logging.INFO)

    if not logger.handlers:
        try:
            log_dir = mkdirs("logs")
            log_path = os.path.join(log_dir, f"{bpName}.log")
            handler = RotatingFileHandler(log_path, maxBytes=maxBytes, backupCount=backupCount)
            handler.setLevel(logging.INFO)

            fmt = "%(asctime)s - %(levelname)s - %(pathname)s - %(message)s"
            formatter = logging.Formatter(fmt)
            handler.setFormatter(formatter)
            logger.addHandler(handler)

            console = logging.StreamHandler()
            console.setLevel(logging.INFO)
            console.setFormatter(formatter)
            logger.addHandler(console)
        except Exception as e:
            print(f"{e}")
    return logger

def _find_caller_frame_index():
    """
    Scan up the call stack until we find a frame whose module is NOT logging_utils.
    Return that index in inspect.stack().
    """
    for idx, frame_info in enumerate(inspect.stack()):
        # Ignore the very first frame (idx=0), which is this function itself.
        if idx == 0:
            continue
        module = inspect.getmodule(frame_info.frame)
        # If module is None (e.g. interactive), skip it;
        # else get module.__name__ and compare:
        module_name = module.__name__ if module else None

        # Replace 'yourpackage.logging_utils' with whatever your actual module path is:
        if module_name != __name__ and not module_name.startswith("logging"):
            # We found a frame that isn’t in this helper module or the stdlib logging.
            return idx
    # Fallback to 1 (the immediate caller) if nothing else matches:
    return 1
